{
This unit allows Turbo/Borland Pascal programs to read text files
with different newline character combinations than the CR/LF
combination used under Dos. Currently, files with only CR or only LF
as newline can be read just like files with CR/LF via the usual Read
and Readln procedures. The newline characters supported can be
changed in the implementation part of the unit. The unit was written
for BP 7.0.

Disclaimer:
This code is provided "as is", and comes WITHOUT ANY WARRANTY, expressed or
implied; without even the implied warranty of fitness for a particular
purpose. I shall not be liable for any damages, whether direct, indirect,
special, incidental or consequential arising from the use of the software or
from the inability to use the software or from a failure of the software to
operate in a manner desired by the user.
In short: Use this code totally at your own risk!

This unit provides the following function, followed by a short demo
program how to use the unit:

function AssignCrLf (var f : Text) : Boolean;

- Must be called after assigning and resetting f, before reading from it.
- Returns False if f is not open for reading or AssignCrLf had already been
  called on f.
- Works completely transparent to the program. All single CR or LF chars read
  are replaced by a complete CR/LF pair.
- Can be used on many files at the same time (not tested, though, but since
  there are no globals, it should be no problem).
- Needs heap memory as much as BufSize (usually 128 bytes).
  Memory is freed when f is closed.

Copyright 1996-99 by Frank Heckenbach,
frank(at)pascal.gnu.de or fjf(at)gmx.de

This library is free software; you can redistribute it and/or modify
it under the terms of the GNU Library General Public License as
published by the Free Software Foundation, version 2.

This library is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this library, see the file COPYING.LIB; if not,
write to the Free Software Foundation, Inc., 675 Mass Ave,
Cambridge, MA 02139, USA.
}

unit CrLf;

interface

{$X+}

uses Dos;

function AssignCrLf (var f : Text) : Boolean;

implementation

const
  MagicID = $8df5;
  CR = #13;
  LF = #10;

type
  TIOFunc = function (var f : TextRec) : Integer;
  TUserRec = record
    Magic, TempBufLength : Word;
    TempBuf : PChar;
    OldRead, OldClose : TIOFunc
  end;

function CrLfRead (var f : TextRec) : Integer; far;
var
  c : Word;
  Ch, OldCh : Char;
  d : PChar;
  Flag : Boolean;

  procedure Put (Ch : Char);
  begin
    d^ := Ch;
    Inc (d);
    with f, TUserRec (UserData) do
      if d >= BufPtr^ + BufSize then
        begin
          Flag := True;
          d := TempBuf
        end
  end;

begin
  with f, TUserRec (UserData) do
    if Magic = MagicID
      then
        if TempBufLength <> 0
          then
            begin
              Move (TempBuf^, BufPtr^, TempBufLength);
              BufPos := 0;
              BufEnd := TempBufLength;
              TempBuf [0] := TempBuf [TempBufLength];
              TempBufLength := 0;
              CrLfRead := 0
            end
          else
            begin
              CrLfRead := OldRead (f);
              if BufEnd > BufPos then
                begin
                  OldCh := TempBuf [0];
                  Move (BufPtr^ [BufPos], TempBuf^, BufEnd - BufPos);
                  Flag := False;
                  d := BufPtr^;
                  for c := 0 to BufEnd - BufPos - 1 do
                    begin
                      Ch := TempBuf [c];
                      case Ch of
                        CR : if (c < BufEnd - BufPos - 1) and (TempBuf [c + 1] <> LF) then
                             begin
                               Put (CR);
                               Ch := LF
                             end;
                        LF : if OldCh <> CR then Put (CR)
                      end;
                      Put (Ch);
                      OldCh := Ch
                    end;
                  BufPos := 0;
                  if Flag
                    then
                      begin
                        BufEnd := BufSize;
                        TempBufLength := d - TempBuf
                      end
                    else
                      begin
                        BufEnd := d - BufPtr^;
                        TempBufLength := 0
                      end
                end
            end
      else CrLfRead := 149
end;

function CrLfClose (var f : TextRec) : Integer; far;
begin
  with f, TUserRec (UserData) do
    if Magic = MagicID
      then
        begin
          FreeMem (TempBuf, BufSize);
          CrLfClose := OldClose (f)
        end
      else CrLfClose := 149
end;

function AssignCrLf (var f : Text) : Boolean;
begin
  with TextRec (f), TUserRec (UserData) do
    if (Mode <> FmInput) or (Magic = MagicID)
      then AssignCrLf := False
      else
        begin
          Magic := MagicID;
          @OldRead := InOutFunc;
          InOutFunc := @CrLfRead;
          @OldClose := CloseFunc;
          CloseFunc := @CrLfClose;
          GetMem (TempBuf, BufSize);
          TempBuf [0] := #0;
          TempBufLength := 0;
          AssignCrLf := True
        end
end;

end.
