{
This unit allows Turbo/Borland Pascal programs to log text files,
i.e. all text that is read from or written to the file is copied to
the log file. The unit was written for BP 7.0.

This unit provides the following two functions:

function AssignLogFile (var f, LogFile : Text) : Boolean;

- Starts logging f onto LogFile.
- Must be called after assigning and opening (Reset/Rewrite/Append) f, before
  reading from f or writing to f.
- Returns False if f is not open for reading or writing or AssignLogFile had
  already been called on f.
- LogFile must have been opened for output (Rewrite or Append) before calling
  AssignLogFile, and must be closed after closing f or calling UnAssignLogFile
  on f, but LogFile MUST NOT be closed before closing f. In the special case
  of f being Input or Output - which are not closed by the program - LogFile
  may be closed after the last I/O operation on f.
- This unit does NOT assign, open or close LogFile in any case.
- Can be used on many files at the same time.
- The same LogFile can be used for many files at the same time, but for each
  file there can be only one LogFile. However, the LogFile can be LogFiled
  again, so there can be a chain of log files.

function UnAssignLogFile (var f : Text) : Boolean;

- Stops logging f.
- Can be called any time after LogFiling f, before closing f.
- Does not have to be called after LogFiling f. Simply closing f is
  sufficient.
- Returns False if f is not open for reading or writing or AssignLogFile had
  not been called on f.

Copyright 1996-99 by Frank Heckenbach,
frank(at)pascal.gnu.de or fjf(at)gmx.de

This library is free software; you can redistribute it and/or modify
it under the terms of the GNU Library General Public License as
published by the Free Software Foundation, version 2.

This library is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this library, see the file COPYING.LIB; if not,
write to the Free Software Foundation, Inc., 675 Mass Ave,
Cambridge, MA 02139, USA.
}

unit TextLog;

interface

{$X+}

function AssignLogFile (var f, LogFile : Text) : Boolean;
function UnAssignLogFile (var f : Text) : Boolean;

implementation

{ Copied from WinDos for TFDD handling }
const
  fmClosed = $d7b0;
  fmInput  = $d7b1;
  fmOutput = $d7b2;
  fmInOut  = $d7b3;

type
  PTextBuf = ^TTextBuf;
  TTextBuf = array [0 .. 127] of Char;
  TTextRec = record
    Handle, Mode, BufSize, private, BufPos, BufEnd : Word;
    BufPtr : PTextBuf;
    OpenFunc, InOutFunc, FlushFunc, CloseFunc : Pointer;
    UserData : array [1 .. 16] of Byte;
    Name : array [0 .. 79] of Char;
    Buffer : TTextBuf
  end;

  PText = ^Text;
  TIOFunc = function (var f : TTextRec) : Integer;
  TUserRec = record
    Magic, Fill : Word;
    PLogFile : PText;
    OldIO, OldFlush : TIOFunc
  end;

const
  MagicID = $8df6;

procedure Log (o, c : Word; var Buf : TTextBuf; pf : PText);
var s : String;
begin
  if not Assigned (pf) then Exit;
  s [0] := #$ff;
  while c > 0 do
    begin
      if c < Length (s) then s [0] := Chr (c);
      Move (Buf [o], s [1], Length (s));
      Inc (o, Length (s));
      Dec (c, Length (s));
      Write (pf^, s)
    end
end;

function LogIO (var f : TTextRec) : Integer; far;
begin
  with f, TUserRec (UserData) do
    if Magic = MagicID
      then
        if Mode = FmInput
          then
            begin
              LogIO := OldIO (f);
              Log (BufPos, BufEnd - BufPos, BufPtr^, PLogFile);
              if InOutRes <> 0 then LogIO := IOResult
            end
          else
            begin
              Log (0, BufPos, BufPtr^, PLogFile);
              if InOutRes <> 0 then LogIO := IOResult else LogIO := OldIO (f)
            end
      else LogIO := 149
end;

function LogFlush (var f : TTextRec) : Integer; far;
begin
  with f, TUserRec (UserData) do
    if Magic = MagicID
      then
        begin
          Log (0, BufPos, BufPtr^, PLogFile);
          if InOutRes <> 0
            then LogFlush := IOResult
            else LogFlush := OldFlush (f)
        end
      else LogFlush := 149
end;

function AssignLogFile (var f, LogFile : Text) : Boolean;
begin
  with TTextRec (f), TUserRec (UserData) do
    if ((Mode <> FmInput) and (Mode <> FmOutput)) or (Magic = MagicID)
      then AssignLogFile := False
      else
        begin
          Magic := MagicID;
          @OldIO := InOutFunc;
          InOutFunc := @LogIO;
          if (Mode = FmOutput) and Assigned (FlushFunc) then
            begin
              @OldFlush := FlushFunc;
              FlushFunc := @LogFlush
            end;
          PLogFile := @LogFile;
          AssignLogFile := True
        end
end;

function UnAssignLogFile (var f : Text) : Boolean;
begin
  with TTextRec (f), TUserRec (UserData) do
    if ((Mode <> FmInput) and (Mode <> FmOutput)) or (Magic <> MagicID)
      then UnAssignLogFile := False
      else
        begin
          Flush (PLogFile^);
          PLogFile := nil; { This, together with the check in Log, is important
                             if UnAssignLogFile is called while an I/O on f is
                             active (easily possible under Windoze). }
          Magic := 0;
          InOutFunc := @OldIO;
          if (Mode = FmOutput) and Assigned (FlushFunc) then FlushFunc := @OldFlush;
          UnAssignLogFile := True
        end
end;

end.
